import datetime
import glob
import os
import os.path
from typing import Optional

import fire
import fire.core
from PIL import Image

from anshitsu.retouch import Retouch


def process(
    path: str,
    colorautoadjust: bool = False,
    colorstretch: bool = False,
    grayscale: bool = False,
    invert: bool = False,
    tosaka: Optional[float] = None,
) -> str:
    """
    Process Runnner for Command Line Interface

    This utility converts the colors of images such as photos.

    If you specify a directory path, it will convert
    the image files in the specified directory.
    If you specify a file path, it will convert the specified file.
    If you specify an option, the specified conversion will be performed.

    Tosaka mode is a mode that expresses the preference of
    Tosaka-senpai, a character in "Kyūkyoku Chōjin R",
    for "photos taken with Tri-X that look like they were
    burned onto No. 4 or No. 5 photographic paper".
    Only use floating-point numbers when using this mode;
    numbers around 2.4 will make it look right.

    Args:
        path (str): Directory or File Path
        colorautoadjust (bool, optional): Use colorautoadjust algorithm. Defaults to False.
        colorstretch (bool, optional): Use colorstretch algorithm. Defaults to False.
        grayscale (bool, optional): Convert to grayscale. Defaults to False.
        invert (bool, optional): Invert color. Defaults to False.
        tosaka (Optional[float], optional): Use Tosaka mode. Defaults to None.

    Raises:
        fire.core.FireError: Error that occurs when the specified string is not a path.

    Returns:
        str: Message.
    """
    types = ("*.jpg", "*.JPG", "*.jpeg", "*.JPEG", "*.png", "*.PNG")
    files_glob = []
    return_path = ""
    now_s = datetime.datetime.now()
    if os.path.isdir(path):
        for files in types:
            files_glob.extend(
                glob.glob(
                    os.path.join(path, "**", files),
                    recursive=True,
                )
            )
        return_path = path
    elif os.path.isfile(path):
        files_glob.extend(glob.glob(path))
        return_path = os.path.abspath(os.path.join(path, os.pardir))
    else:
        raise fire.core.FireError("A non-path string was passed.")
    for i, file in enumerate(files_glob):
        image = Image.open(file)
        retouch = Retouch(
            image=image,
            colorautoadjust=colorautoadjust,
            colorstretch=colorstretch,
            grayscale=grayscale,
            invert=invert,
            tosaka=tosaka,
        )
        saved_image = retouch.process()
        os.makedirs(os.path.join(return_path, "out"), exist_ok=True)
        saved_image.save(
            os.path.join(
                return_path,
                "out",
                "{0}_{1}.jpg".format(now_s.strftime("%Y-%m-%d_%H-%M-%S"), (i + 1)),
            ),
            quality=100,
            subsampling=0,
        )
        print("{0}/{1} done!".format((i + 1), str(len(files_glob))))

    return "The process was completed successfully."



from typing import Optional

import colorcorrect.algorithm as cca
import numpy as np
from colorcorrect.util import from_pil, to_pil
from PIL import Image, ImageEnhance, ImageOps


class Retouch:
    """
    Perform retouching.

    Passing an image and options to the constructor will convert the specified image.
    """

    def __init__(
        self,
        image: Image,
        colorautoadjust: bool = False,
        colorstretch: bool = False,
        grayscale: bool = False,
        invert: bool = False,
        tosaka: Optional[float] = None,
    ) -> None:
        """
        __init__ constructor.

        Args:
            image (Image): Image file.
            colorautoadjust (bool, optional): Use colorautoadjust algorithm. Defaults to False.
            colorstretch (bool, optional): Use colorstretch algorithm. Defaults to False.
            grayscale (bool, optional): Convert to grayscale. Defaults to False.
            invert (bool, optional): Invert color. Defaults to False.
            tosaka (Optional[float], optional): Use Tosaka mode. Defaults to None.
        """
        self.image = image
        self.colorautoadjust = colorautoadjust
        self.colorstretch = colorstretch
        self.grayscale = grayscale
        self.invert = invert
        self.tosaka = tosaka

    def process(self) -> Image:
        self.image = self.__rgba_convert()

        if self.invert:
            self.image = self.__invert()

        if self.colorautoadjust:
            self.image = self.__colorautoadjust()

        if self.colorstretch:
            self.image = self.__colorstretch()

        if self.grayscale:
            self.image = self.__grayscale()

        if self.tosaka is not None:
            self.image = self.__tosaka()

        return self.image

    def __colorautoadjust(self) -> Image:
        """
        __colorautoadjust

        Use colorautoadjust algorithm.

        Args:
            image (Image): base image.

        Returns:
            Image: processed image.
        """
        if self.image.mode == "L" or self.image.mode == "LA":
            return self.image
        return to_pil(cca.automatic_color_equalization(from_pil(self.image)))

    def __colorstretch(self) -> Image:
        """
        __colorstretch

        Use colorstretch algorithm.

        Args:
            image (Image): base image.

        Returns:
            Image: processed image.
        """
        if self.image.mode == "L" or self.image.mode == "LA":
            return self.image
        return to_pil(cca.stretch(cca.grey_world(from_pil(self.image))))

    def __grayscale(self) -> Image:
        """
        __grayscale

        Convert to grayscale.

        Args:
            image (Image): base image.

        Returns:
            Image: processed image.
        """
        if self.image.mode == "L":
            return self.image

        rgb = np.array(self.image, dtype="float32")

        rgbL = pow(rgb / 255.0, 2.2)
        r, g, b = rgbL[:, :, 0], rgbL[:, :, 1], rgbL[:, :, 2]
        grayL = 0.299 * r + 0.587 * g + 0.114 * b  # BT.601
        gray = pow(grayL, 1.0 / 2.2) * 255
        self.image = Image.fromarray(gray.astype("uint8"))

        return self.image

    def __invert(self) -> Image:
        """
        __invert

        Invert color.

        Args:
            image (Image): base image.

        Returns:
            Image: processed image.
        """
        return ImageOps.invert(self.image)

    def __tosaka(self) -> Image:
        """
        __tosaka

        Use Tosaka mode.

        Tosaka mode is a mode that expresses the preference of Tosaka-senpai, a character in "Kyūkyoku Chōjin R", for "photos taken with Tri-X that look like they were burned onto No. 4 or No. 5 photographic paper". Only use floating-point numbers when using this mode; numbers around 2.4 will make it look right.

        Args:
            image (Image): base image.
            tosaka (float): [description]

        Returns:
            Image: processed image.
        """
        if self.image.mode != "L":
            self.image = self.__grayscale()
        imageC = ImageEnhance.Contrast(self.image)
        self.image = imageC.enhance(self.tosaka)
        return self.image

    def __rgba_convert(self) -> Image:
        """
        __rgba_convert

        Converts image data that contains transparency to image data that does not contain transparency.

        Args:
            image (Image): base image.

        Returns:
            Image: processed image.
        """
        if self.image.mode == "RGBA":
            self.image.load()
            background = Image.new("RGB", self.image.size, (255, 255, 255))
            background.paste(self.image, mask=self.image.split()[3])
            self.image = background
        if self.image.mode == "LA":
            self.image.load()
            background = Image.new("L", self.image.size, (255))
            background.paste(self.image, mask=self.image.split()[1])
            self.image = background
        return self.image



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['anshitsu']

package_data = \
{'': ['*']}

install_requires = \
['Pillow>=8.3.1,<9.0.0',
 'colorcorrect>=0.9.1,<0.10.0',
 'fire>=0.4.0,<0.5.0',
 'numpy>=1.21.0,<2.0.0']

entry_points = \
{'console_scripts': ['anshitsu = anshitsu.cli:main']}

setup_kwargs = {
    'name': 'anshitsu',
    'version': '1.1.0',
    'description': 'A tiny digital photographic utility.',
    'long_description': '# Anshitsu\n\n[![Testing](https://github.com/huideyeren/anshitsu/actions/workflows/testing.yml/badge.svg)](https://github.com/huideyeren/anshitsu/actions/workflows/testing.yml)\n\n[![codecov](https://codecov.io/gh/huideyeren/anshitsu/branch/main/graph/badge.svg?token=ZYRX8NBTLQ)](https://codecov.io/gh/huideyeren/anshitsu)\n\nA tiny digital photographic utility.\n\n"Anshitsu" means a darkroom in Japanese.\n\n## Usage\n\n```\nINFO: Showing help with the command \'anshitsu -- --help\'.\n\nNAME\n    anshitsu - Process Runnner for Command Line Interface\n\nSYNOPSIS\n    anshitsu PATH <flags>\n\nDESCRIPTION\n    This utility converts the colors of images such as photos.\n\n    If you specify a directory path, it will convert\n    the image files in the specified directory.\n    If you specify a file path, it will convert the specified file.\n    If you specify an option, the specified conversion will be performed.\n\n    Tosaka mode is a mode that expresses the preference of\n    Tosaka-senpai, a character in "Kyūkyoku Chōjin R",\n    for "photos taken with Tri-X that look like they were\n    burned onto No. 4 or No. 5 photographic paper".\n    Only use floating-point numbers when using this mode;\n    numbers around 2.4 will make it look right.\n\nPOSITIONAL ARGUMENTS\n    PATH\n        Type: str\n        Directory or File Path\n\nFLAGS\n    --colorautoadjust=COLORAUTOADJUST\n        Type: bool\n        Default: False\n        Use colorautoadjust algorithm. Defaults to False.\n    --colorstretch=COLORSTRETCH\n        Type: bool\n        Default: False\n        Use colorstretch algorithm. Defaults to False.\n    --grayscale=GRAYSCALE\n        Type: bool\n        Default: False\n        Convert to grayscale. Defaults to False.\n    --invert=INVERT\n        Type: bool\n        Default: False\n        Invert color. Defaults to False.\n    --tosaka=TOSAKA\n        Type: Optional[typing.Un...\n        Default: None\n        Use Tosaka mode. Defaults to None.\n\nNOTES\n    You can also use flags syntax for POSITIONAL ARGUMENTS\n```\n\n\n\n## Algorithm\n\n### RGBA to RGB Convert\n\nConverts an image that contains Alpha, such as RGBA, to image data that does not contain Alpha.\nTransparent areas will be filled with white.\n\n### invert\n\nColor inversion by pillow.\n\n### colorautoajust\n\nUsing "automatic color equalization" algorithm.\n\n(References)\n\nA. Rizzi, C. Gatta and D. Marini, "A new algorithm for unsupervised global and local color correction.", Pattern Recognition Letters, vol. 24, no. 11, 2003.\n\n### colorstretch\n\nUsing "stretch" algorithm after "gray world" algorithm.\n\n(References)\n\nD. Nikitenko, M. Wirth and K. Trudel, "Applicability Of White-Balancing Algorithms to Restoring Faded Colour Slides: An Empirical Evaluation.", Journal of Multimedia, vol. 3, no. 5, 2008.\n\n### grayscale\n\nI implemented it based on the method described in this article.\n\n[Python でグレースケール(grayscale)化](https://qiita.com/yoya/items/dba7c40b31f832e9bc2a#pilpillow-%E3%81%A7%E3%82%B0%E3%83%AC%E3%83%BC%E3%82%B9%E3%82%B1%E3%83%BC%E3%83%AB%E5%8C%96-numpy-%E3%81%A7%E4%BD%8E%E8%BC%9D%E5%BA%A6%E5%AF%BE%E5%BF%9C)\n\nNote: This article is written in Japanese.\n\n### Tosaka mode\n\nTosaka mode is a mode that expresses the preference of Tosaka-senpai, a character in "Kyūkyoku Chōjin R", for "photos taken with Tri-X that look like they were burned onto No. 4 or No. 5 photographic paper".\n\nOnly use floating-point numbers when using this mode; numbers around 2.4 will make it look right.\n\nWhen this mode is specified, color images will also be converted to grayscale.\n\n## Special Thanks\n\nWe are using the following libraries.\n\n[shunsukeaihara/colorcorrect](https://github.com/shunsukeaihara/colorcorrect)\n',
    'author': 'Iosif Takakura',
    'author_email': 'iosif@huideyeren.info',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/huideyeren',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
